#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define boolean  int
#define VERDADEIRO 1
#define FALSO      0

/*
			     =============
			      OBSERVAOES
			     =============

     Em todas as funoes deste arquivo:

   - o primeiro caracter de um string X e' o caracter X[0].

   - o parametro de retorno de uma funcao, se for um string, precisa ser
     suficientemente grande para conter a resposta.  O  tamanho  exigido
     no parametro de retorno deve poder ser calculado usando-se a funcao
     strlen() ou a macro TAMANHO() definida a seguir.
*/

/* ------------------------------------------------------------------------ */

/* Calcula o tamanho do string s. E' o mesmo que tam = strlen(s).
   Ex.: TAMANHO("teste", tam)   ---> tam = 5                       */

#define TAMANHO(s, tam)       tam = 0; if (s[0]) while(s[++tam])

/* ------------------------------------------------------------------------ */

char *Repete(char *x, char caracter, int n) {

/* Retorna o string formado pelo caracter dado repetido n vezes.
   Ex.: Repete(x, '+', 6) = "++++++"  */

   register int i = 0;
   if (n < 0) n = -n;
   for (i = 0; i < n; i++) x[i] = caracter;
   x[i] = '\0';
   return x;
}

/* ------------------------------------------------------------------------ */

char *EliminaCaracter(char *x, char car_x) {

/* Elimina todas as ocorrencias do caracter dado do string x.
   Ex.: EliminaEspacos(".1..2..3.", '.') = "123"    */

  register int i, j = 0, comprimento;
  char *aux;

  TAMANHO(x, comprimento);
  if ((aux = (char *) malloc(sizeof(char)*(comprimento + 1))) == NULL)
    return x;

  for (i = 0; i < comprimento; i++)
    if (x[i] != car_x) aux[j++] = x[i];
  aux[j] = '\0';
  strcpy(x, aux);
  free(aux);
  return x;
}

/* ======================== COMPARACAO DE STRINGS ========================= */

/* Compara os strings s1 e s2. Retorna 1 se forem identicos ou 0 forem
   distintos.
   Ex.: COMPSTR("teste", "teste") = 1
	COMPSTR("teste", "TESTE") = 0                  */

#define COMPSTR(s1, s2)  (Compara(s1, s2) < 0)

/* ------------------------------------------------------------------------ */

int Compara(char *s1, char *s2) {

/* Compara os strings s1 e s2 e retorna a posicao dos primeiros caracteres
   que forem diferentes. Retorna -1 se os strings forem identicos.
   Ex.: Compara("um", "dois") = 0
	Compara("teste", "testE") = 4
	Compara("teste", "teste") = -1
	Compara("Alo", "Alo!!!") = 3     */

  register int i = 0;

  while (s1[i] != '\0' && s2[i] != '\0' && s1[i] == s2[i]) i++;
  if (s1[i] == '\0' && s2[i] == '\0') return -1;
  else return i;
}

/* ------------------------------------------------------------------------ */

/* Compara os n primeiros caracteres dos strings s1 e s2. Retorna 1 se
   forem identicos ou 0 forem distintos.
   Ex.: COMPSTRN("teste", "teste2...", 5) = 1
	COMPSTRN("teste", "Teste2...", 5) = 0                  */

#define COMPSTRN(s1, s2, n) (ComparaN(s1, s2, n) < 0)

/* ------------------------------------------------------------------------ */

int ComparaN(char *s1, char *s2, int n) {

/* Compara os n primeiros caracteres  dos strings s1 e s2 e retorna
   a posicao dos primeiros caracteres que forem diferentes. Retorna
   -1 se os strings coincidirem nos n primeiros caracteres.
   Ex.: ComparaN("um", "dois", 10) = 0
	ComparaN("teste", "testE", 4) = -1
	ComparaN("Alo", "Alo!!!", 5) = 3
	ComparaN("Alo", "Alo!!!", 3) = -1     */

  register int i = 0;

  while (i < n && s1[i] != '\0' && s2[i] != '\0' && s1[i] == s2[i]) i++;
  if (i == n || (s1[i] == '\0' && s2[i] == '\0')) return -1;
  else return i;
}

/* ------------------------------------------------------------------------ */

/* Compara os strings s1 e s2.  Retorna 1 se forem identicos  ou  0  forem
   distintos. Letras maiusculas sao consideradas identicas `as respectivas
   letras minusculas.
   Ex.: COMPSTRI("teste", "TESTE") = 1       */

#define COMPSTRI(s1, s2) (ComparaI(s1, s2) < 0)

/* ------------------------------------------------------------------------ */

int ComparaI(char *s1, char *s2) {

/* Compara os strings s1 e s2 e retorna a posicao dos primeiros caracteres
   que forem diferentes. Retorna -1 se os strings forem identicos.  Letras
   maiusculas sao consideradas identicas `as respectivas letras minusculas.
   Ex.: ComparaI("um", "dois") = 0
	ComparaI("teste", "TESTE") = -1
	ComparaI("teste", "teste") = -1
	ComparaI("Alo", "Alo!!!") = 3     */

  register int i = 0;

  while (s1[i] != '\0' && s2[i] != '\0' &&
				    toupper(s1[i]) == toupper(s2[i])) i++;
  if (s1[i] == '\0' && s2[i] == '\0') return -1;
  else return i;
}

/* ------------------------------------------------------------------------ */

/* Compara os n primeiros caracteres dos strings s1 e s2. Retorna 1 se
   forem identicos ou 0 forem distintos. Nao  e' feita distincao entre
   letras maiusculas e as respectivas letras minusculas.
   Ex.: COMPSTRNI("teste", "teste2...", 5) = 1
	COMPSTRNI("teste", "TESTE2...", 5) = 1                  */

#define COMPSTRNI(s1, s2, n) (ComparaNI(s1, s2, n) < 0)

/* ------------------------------------------------------------------------ */

int ComparaNI(char *s1, char *s2, int n) {

/* Compara os n primeiros caracteres dos strings  s1  e  s2 e retorna a
   posicao dos primeiros caracteres que forem diferentes. Retorna -1 se
   os strings coincidirem nos n primeiros caracteres. Letras maiusculas
   sao consideradas identicas `as respectivas letras minusculas.
   Ex.: ComparaNI("um", "dois", 10) = 0
	ComparaNI("teste", "TESTE", 4) = -1
	ComparaNI("Alo", "Alo!!!", 5) = 3
	ComparaNI("Alo", "Alo!!!", 3) = -1     */

  register int i = 0;

  while (i < n && s1[i] != '\0' && s2[i] != '\0' &&
				    toupper(s1[i]) == toupper(s2[i])) i++;
  if (i == n || (s1[i] == '\0' && s2[i] == '\0')) return -1;
  else return i;
}

/* =================== CONTAGEM OU POSICAO DE ELEMENTOS =================== */

/* ------------------------------------------------------------------------ */

/* Verifica se o substring s1 ocorre em s2.
   Ex.: STR_PERTENCE("ghi", "abcdefghij") = 1     */

#define STR_PERTENCE(s1, s2) (PrimeiraOcorrencia(s1, s2, 0) >= 0)

/* ------------------------------------------------------------------------ */

int PrimeiraOcorrencia(char *substring, char *string, int posicao) {

/* Retorna a primeira ocorrencia de um substring em um string a partir
   de uma certa posicao. Retorna -1 se o substring nao for encontrado.
   Ex.: PrimeiraOcorrencia("de", "Universidade Federal", 0) = 10
	PrimeiraOcorrencia("de", "Universidade Federal", 12) = 15
	PrimeiraOcorrencia("xy", "Universidade Federal", 0) = -1        */

  register int i, j;

  for (i = posicao; string[i]; i++)
    for (j = 0; !substring[j] || string[i + j] == substring[j]; j++)
      if (!substring[j]) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int UltimaOcorrenciaI(char *substring, char *string, int posicao) {

/* Retorna a ultima ocorrencia de um substring em um string antes de uma
   certa posicao.  Se  posicao < 0, sera'  considerado  todo  o  string.
   Se o substring nao ocorrer dentro  do intervalo indicado sera' retor-
   nado o valor -1. Letras maiusculas sao consideradas identicas `as
   respectivas letras minusculas.
   Ex.: UltimaOcorrenciaI("CA", "Matematica Aplicada", -2) = 15
	UltimaOcorrenciaI("Ca", "Matematica Aplicada", 17) = 15
	UltimaOcorrenciaI("ca", "Matematica Aplicada", 5) = -1
	UltimaOcorrenciaI("cA", "Matematica Aplicada", 8) = 8      */

  register int i, j;

  if (posicao < 0) {
    posicao = 0;
    while(string[posicao]) posicao++;
  }
  for (i = posicao; i >= 0;  i--)
    for (j = 0; !substring[j] || toupper(string[i + j])
			      == toupper(substring[j]); j++)
      if (!substring[j]) return i;
  return -1;

}

/* ------------------------------------------------------------------------ */

/* Verifica se o substring s1 ocorre em s2. Letras maiusculas sao identifi-
   cadas com as respectivas letras minusculas.
   Ex.: STR_PERTENCE_I("GHI", "abcdefghij") = 1      */

#define STR_PERTENCE_I(s1, s2) (PrimeiraOcorrenciaI(s1, s2, 0) >= 0)

/* ------------------------------------------------------------------------ */

int PrimeiraOcorrenciaI(char *substring, char *string, int posicao) {

/* Retorna a primeira ocorrencia de um substring em um string a partir
   de uma certa posicao. Retorna -1 se o substring nao for encontrado.
   Letras maiusculas sao consideradas identicas `as respectivas letras
   minusculas.
   Ex.: PrimeiraOcorrenciaI("DE", "Universidade Federal", 0) = 10
	PrimeiraOcorrenciaI("DE", "Universidade Federal", 12) = 15
	PrimeiraOcorrenciaI("xy", "Universidade Federal", 0) = -1        */

  register int i, j;

  for (i = posicao; string[i]; i++)
    for (j = 0; !substring[j] || toupper(string[i + j])
			      == toupper(substring[j]); j++)
      if (!substring[j]) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int Pos(char x, char *s) {

/* Retorna a primeira posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s.
   Ex.: Pos('a', "Matematica") = 1           */

  register int i;

  for(i = 0; s[i]; i++)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int PosI(char x, char *s) {

/* Retorna a primeira posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s. Nao e' feita distincao entre letras minusculas
   e maiusculas.
   Ex.: PosI('A', "Matematica") = 1           */

  register int i;

  for(i = 0; s[i]; i++)
    if (toupper(s[i]) == toupper(x)) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int PosUlt(char x, char *s) {

/* Retorna a ultima posicao do caracter x no string s. Retorna -1 se
   x nao pertencer a s.
   Ex.: PosUlt('a', "Matematica") = 9        */

  register int i, tam;

  TAMANHO(s, tam);
  for(i = tam; i >= 0; i--)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

int PosN(char x, char *s, int n) {

/* Retorna a posicao do caracter x no string s situado a partir na n-esima
   posicao . Retorna -1 se x nao pertencer a x (apo's a n-esima posicao).
   Ex.: PosN('a', "Matematica", 3) = 5
	PosN('m', "Matematica", 6) = -1                                  */

  register int i, tam;

  TAMANHO(s, tam);
  for(i = n; i < tam; i++)
    if (s[i] == x) return i;
  return -1;
}

/* ------------------------------------------------------------------------ */

/* ================== TROCA OU SUBSTITUICAO DE CARACTERES ================= */

char *Traduz(char *s, char *tabela_entrada, char *tabela_saida) {

/* Retorna s apos a troca de caracteres de s em tabela_entrada[] pelos
   respectivos  caracteres  em tabela_saida[]. As tabelas devem ser do
   mesmo tamanho.
   Ex.: Traduz("PROGRAM", "GXAR", "BQEL") =  "PROBLEM"
	Traduz("+-+-Teste%^%^", "+-%^", "....") = "....Teste...."  */

  int i = 0, j, tam1, tam2;

  TAMANHO(tabela_entrada, tam1);
  TAMANHO(tabela_saida, tam2);
  if (tam1 != tam2) return NULL;

  while (s[i]) {
    for (j = 0; j < tam1; j++)
      if (s[i] == tabela_entrada[j]) s[i] = tabela_saida[j];
    i++;
  }
  return s;
}

/* ------------------------------------------------------------------------ */

char *Substitui(char *x, char *a, char *b) {

/* Substitui "a" por "b" em "x" .
   Ex.: Substitui("abcxyz ABC", "abc", "***") = "***xyz ABC"    */

  int tam_x, tam_a, coincide, i = 0, j, k;
  char *aux;

  TAMANHO(x, tam_x);
  TAMANHO(a, tam_a);
  if (tam_x < tam_a) return x;
  if ((aux = (char *) malloc(sizeof(char)*(tam_x + 1))) == NULL) return x;

  aux[0] = '\0';
  while (i <= tam_x) {
    j = i;
    k = 0;
    coincide = 0;

    while ((k < tam_a) && (j < tam_x) && (a[k] == x[j])) {
      coincide = coincide + 1;
      k++;
      j++;
    }

    if (coincide == tam_a) {
      strcat(aux, b);
      i += tam_a;
    } else {
      int n;
      n = strlen(aux);
      aux[n++] = x[i]; aux[n] = '\0';
      i++;
    }
  }

  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *SubstituiI(char *x, char *a, char *b) {

/* Substitui "a" por "b" em "x" . Letras maiusculas sao identificadas
   com as respectivas letras minusculas.
   Ex.: SubstituiI("abcxyz ABC", "abc", "***") = "***xyz ***"    */

  int tam_x, tam_a, coincide, i = 0, j, k;
  char *aux;

  TAMANHO(a, tam_a);
  TAMANHO(x, tam_x);
  if (tam_x < tam_a) return x;
  if ((aux = (char *) malloc(sizeof(char)*(tam_x + 1))) == NULL) return x;

  aux[0] = '\0';
  while (i <= tam_x) {
    j = i;
    k = 0; coincide = 0;

    while ((k < tam_a) && (j < tam_x) && (toupper(a[k]) == toupper(x[j]))){
      coincide = coincide + 1;
      k++;
      j++;
    }

    if (coincide == tam_a) {
      strcat(aux, b);
      i += tam_a;
    } else {
      int n;
      n = strlen(aux);
      aux[n++] = x[i]; aux[n] = '\0';
      i++;
    }
  }

  strcpy(x, aux);
  free(aux);
  return x;
}

/* ------------------------------------------------------------------------ */

char *SubstituiChar(char *x, char a, char b) {

/* Substitui 'a' por 'b' em "x" .
   Ex.: SubstituiChar("abcxyz ABC", 'x', '*') = "abc*yz ABC"    */

  register int i = 0;

  while (x[i] != '\0') {
    if (x[i] == a) x[i] = b;
    i++;
  }
  return x;
}

/* ------------------------------------------------------------------------ */

char *SubstituiCharI(char *x, char a, char b) {

/* Substitui 'a' por 'b' em "x", ignorando se a e' minuscula ou maiuscula.
   Ex.: SubstituiCharI("abcxyz ABC", 'X', '*') = "abc*yz ABC"    */

  register int i = 0;

  while (x[i] != '\0') {
    if (toupper(x[i]) == toupper(a)) x[i] = b;
    i++;
  }
  return x;
}

/* ------------------------------------------------------------------------ */
